@@ -33,11 +33,10 @@ from distutils.dir_util import copy_tree
 from jinja2 import Environment, FileSystemLoader, ChoiceLoader, PrefixLoader
 from jinja2.exceptions import TemplateNotFound
 
-from .image import Image
-from .settings import get_thumb
+from .image import generate_thumbnail
+from .settings import get_thumb, get_orig
 from .pkgmeta import __url__ as sigal_link
 
-INDEX_PAGE = "index.html"
 THEMES_PATH = os.path.normpath(os.path.join(
     os.path.abspath(os.path.dirname(__file__)), 'themes'))
 
@@ -45,14 +44,17 @@ THEMES_PATH = os.path.normpath(os.path.join(
 class Writer(object):
     """Generate html pages for each directory of images."""
 
-    def __init__(self, settings, output_dir, theme=None):
+    def __init__(self, settings, output_dir, theme=None,
+                 template_file="index.html", output_file="index.html"):
         self.settings = settings
         self.output_dir = os.path.abspath(output_dir)
         self.theme = theme or settings['theme']
+        self.template_file = template_file
+        self.output_file = output_file
         self.logger = logging.getLogger(__name__)
 
         # optionally add index.html to the URLs
-        self.url_ext = INDEX_PAGE if settings['index_in_url'] else ''
+        self.url_ext = self.output_file if settings['index_in_url'] else ''
 
         # search the theme in sigal/theme if the given one does not exists
         if not os.path.exists(self.theme):
@@ -60,7 +62,7 @@ class Writer(object):
             if not os.path.exists(self.theme):
                 raise Exception("Impossible to find the theme %s" % self.theme)
 
-        self.logger.info("Theme path : %s", self.theme)
+        self.logger.info("Theme  : %s", self.theme)
         theme_relpath = os.path.join(self.theme, 'templates')
         default_loader = FileSystemLoader(
             os.path.join(THEMES_PATH, 'default', 'templates'))
@@ -75,7 +77,7 @@ class Writer(object):
         )
 
         try:
-            self.template = env.get_template(INDEX_PAGE)
+            self.template = env.get_template(self.template_file)
         except TemplateNotFound:
             self.logger.error(colored.red('ERROR:') +
                               ' The index.html template was not found.')
@@ -113,10 +115,10 @@ class Writer(object):
 
         return reversed(breadcumb)
 
-    def write(self, paths, relpath):
-        """Render the html page."""
+    def generate_context(self, paths, relpath):
+        """Generate the context dict for the given path."""
 
-        path = os.path.join(self.output_dir, relpath)
+        path = os.path.normpath(os.path.join(self.output_dir, relpath))
         index_url = os.path.relpath(self.output_dir, path) + '/' + self.url_ext
         self.logger.info("Output path : %s", path)
 
@@ -132,8 +134,11 @@ class Writer(object):
             ctx['breadcumb'] = self.get_breadcumb(paths, relpath)
 
         for i in paths[relpath]['img']:
-            ctx['images'].append({'file': i,
-                                  'thumb': get_thumb(self.settings, i)})
+            img_ctx = {'file': i,
+                       'thumb': get_thumb(self.settings, i)}
+            if self.settings['keep_orig']:
+                img_ctx['big'] = get_orig(self.settings, i)
+            ctx['images'].append(img_ctx)
 
         for d in paths[relpath]['subdir']:
             dpath = os.path.normpath(os.path.join(relpath, d))
@@ -145,10 +150,11 @@ class Writer(object):
             # generate the thumbnail if it is missing (if
             # settings['make_thumbs'] is False)
             if not os.path.exists(thumb_path):
-                img = Image(os.path.join(self.output_dir, dpath, alb_thumb))
-                img.thumbnail(thumb_path, self.settings['thumb_size'],
-                              fit=self.settings['thumb_fit'],
-                              quality=self.settings['jpg_quality'])
+                source = os.path.join(self.output_dir, dpath, alb_thumb)
+                generate_thumbnail(
+                    source, thumb_path, self.settings['thumb_size'],
+                    fit=self.settings['thumb_fit'],
+                    quality=self.settings['jpg_options']['quality'])
 
             ctx['albums'].append({
                 'url': d + '/' + self.url_ext,
@@ -156,8 +162,14 @@ class Writer(object):
                 'thumb': os.path.join(d, thumb_name)
             })
 
-        # generate html page and save
+        return ctx
+
+    def write(self, paths, relpath):
+        """Generate the HTML page and save it."""
+
+        ctx = self.generate_context(paths, relpath)
         page = self.template.render(paths[relpath], **ctx)
 
-        with codecs.open(os.path.join(path, INDEX_PAGE), 'w', 'utf-8') as f:
+        output_file = os.path.join(self.output_dir, relpath, self.output_file)
+        with codecs.open(output_file, 'w', 'utf-8') as f:
             f.write(page)
